#include <pcl_conversions/pcl_conversions.h>
#include <sensor_msgs/PointCloud2.h>
#include "livox_ros_driver/CustomMsg.h"

typedef pcl::PointXYZINormal PointType;
typedef pcl::PointCloud<PointType> PointCloudXYZI;

ros::Publisher pub_pcl_out0, pub_pcl_out1;
ros::Subscriber sub_livox_msg1, sub_livox_msg2;
uint64_t TO_MERGE_CNT = 1; 
constexpr bool b_dbg_line = false;
int lidar_num_;//雷达数量
std::vector<livox_ros_driver::CustomMsgConstPtr> livox_data;
std::vector<std::string> lidar_id_;//雷达序列号

void LivoxMsgCbk0(const livox_ros_driver::CustomMsgConstPtr& livox_msg_in)
{
  livox_data.push_back(livox_msg_in);
  if (livox_data.size() < TO_MERGE_CNT) return;

  pcl::PointCloud<PointType> pcl_in;

  for (size_t j = 0; j < livox_data.size(); j++) {
    auto& livox_msg = livox_data[j];
    auto time_end = livox_msg->points.back().offset_time;
    for (unsigned int i = 0; i < livox_msg->point_num; ++i) {
      PointType pt;
      pt.x = livox_msg->points[i].x;
      pt.y = livox_msg->points[i].y;
      pt.z = livox_msg->points[i].z;
      float s = livox_msg->points[i].offset_time / (float)time_end;

      pt.intensity = livox_msg->points[i].line +livox_msg->points[i].reflectivity /10000.0 ; // The integer part is line number and the decimal part is timestamp
      pt.curvature = s*0.1;
      pcl_in.push_back(pt);
    }
  }

  unsigned long timebase_ns = livox_data[0]->timebase;
  ros::Time timestamp;
  timestamp.fromNSec(timebase_ns);

  sensor_msgs::PointCloud2 pcl_ros_msg;
  pcl::toROSMsg(pcl_in, pcl_ros_msg);
  pcl_ros_msg.header.stamp.fromNSec(timebase_ns);
  pcl_ros_msg.header.frame_id = "/livox";
  pub_pcl_out0.publish(pcl_ros_msg);
  livox_data.clear();

  std::cout<<"livox/lidar convered finished!"<<std::endl;
}

void LivoxMsgCbk1(const livox_ros_driver::CustomMsgConstPtr& livox_msg_in) 
{
  livox_data.push_back(livox_msg_in);
  if (livox_data.size() < TO_MERGE_CNT) return;

  pcl::PointCloud<PointType> pcl_in;

  for (size_t j = 0; j < livox_data.size(); j++) {
    auto& livox_msg = livox_data[j];
    auto time_end = livox_msg->points.back().offset_time;
    for (unsigned int i = 0; i < livox_msg->point_num; ++i) {
      PointType pt;
      pt.x = livox_msg->points[i].x;
      pt.y = livox_msg->points[i].y;
      pt.z = livox_msg->points[i].z;
      float s = livox_msg->points[i].offset_time / (float)time_end;

      pt.intensity = livox_msg->points[i].line +livox_msg->points[i].reflectivity /10000.0 ; // The integer part is line number and the decimal part is timestamp
      pt.curvature = s*0.1;
      pcl_in.push_back(pt);
    }
  }

  unsigned long timebase_ns = livox_data[0]->timebase;
  ros::Time timestamp;
  timestamp.fromNSec(timebase_ns);

  sensor_msgs::PointCloud2 pcl_ros_msg;
  pcl::toROSMsg(pcl_in, pcl_ros_msg);
  pcl_ros_msg.header.stamp.fromNSec(timebase_ns);
  pcl_ros_msg.header.frame_id = "/livox";
  pub_pcl_out1.publish(pcl_ros_msg);
  livox_data.clear();
}

int main(int argc, char** argv) {
  ros::init(argc, argv, "livox_repub");
  ros::NodeHandle nh;

  ROS_INFO("start livox_repub");

  lidar_num_ = nh.param("lidar_num",3);
  lidar_id_ = nh.param("lidar_id",lidar_id_);
 
  // ros::Subscriber sub_livox_msg1 = lidar_num_ == 1?\
  //      nh.subscribe<livox_ros_driver::CustomMsg>( "/livox/lidars", 100, LivoxMsgCbk0):\
  //      nh.subscribe<livox_ros_driver::CustomMsg>( "/livox/lidars_"+lidar_id_[0], 100, LivoxMsgCbk0);
   
  //  if(lidar_num_ == 1)
  //  {
  // pub_pcl_out0 = nh.advertise<sensor_msgs::PointCloud2>("/livox/lidar", 100);
  //  }

  //  else
  //  {
  //   ros::Subscriber sub_livox_msg2 = nh.subscribe<livox_ros_driver::CustomMsg>( "/livox/lidars_"+lidar_id_[1], 100, LivoxMsgCbk0);
  //   pub_pcl_out0 = nh.advertise<sensor_msgs::PointCloud2>("/livox/lidar_"+lidar_id_[0], 100);
  //   pub_pcl_out1 = nh.advertise<sensor_msgs::PointCloud2>("/livox/lidar_"+lidar_id_[1], 100);
  //  }


  if(lidar_num_ == 1)
    {
    std::cout<<"lidar_num_:"<<lidar_num_<<std::endl;
      sub_livox_msg1 = nh.subscribe<livox_ros_driver::CustomMsg>(
          "/livox/lidars", 100, LivoxMsgCbk0);
      pub_pcl_out0 = nh.advertise<sensor_msgs::PointCloud2>("/livox/lidar", 100);
      std::cout<<"livox/lidar generate!"<<std::endl;
    }
    else 
    {
      sub_livox_msg1 = nh.subscribe<livox_ros_driver::CustomMsg>(
          "/livox/lidars_"+lidar_id_[0], 100, LivoxMsgCbk0);
      pub_pcl_out0 = nh.advertise<sensor_msgs::PointCloud2>("/livox/lidar_"+lidar_id_[0], 100);

      sub_livox_msg2 = nh.subscribe<livox_ros_driver::CustomMsg>(
          "/livox/lidars_"+lidar_id_[1], 100, LivoxMsgCbk1);
      pub_pcl_out1 = nh.advertise<sensor_msgs::PointCloud2>("/livox/lidar_"+lidar_id_[1], 100);
    }


  ros::spin();
}


